import Flutter
import UIKit
import SwiftJWT
/**
 * Dayona
 * Created on Thu Jun 02 2022
 * email : dayonaart@gmail.com
 * Copyright (c) 2022
 */


public class SwiftJwtsonPlugin: NSObject, FlutterPlugin {
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "jwtson", binaryMessenger: registrar.messenger())
        let instance = SwiftJwtsonPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if call.method == "generatePayload" {
            let arg = call.arguments as! NSDictionary
            result(generatePayload(key:arg["key"] as! String,data: arg["data"] as! String))
        }else if call.method == "verifyJwtToken" {
            let arg = call.arguments as! NSDictionary
            result(verifyJWT(signedJWT:arg["data"] as! String, key:arg["key"] as! String))
        }else if call.method == "decodeJWT" {
            let arg = call.arguments as! NSDictionary
            result(decodeJWT(key:arg["key"] as! String, encodedJWT:arg["data"] as! String))
        }  else {
            result(FlutterMethodNotImplemented)
        }

    }
}
private func generatePayload(key:String,data:String) -> String{
    let myClaims = MyClaims(data:data)
    var myJWT = JWT(claims: myClaims)
    let jwtSigner = JWTSigner.hs256(key: Data(key.utf8))
    let signedJWT = try? myJWT.sign(using: jwtSigner)
    return signedJWT ?? "error encode JWT"
}

private func verifyJWT(signedJWT:String,key:String) -> Bool{
    let jwtVerifier = JWTVerifier.hs256(key:  Data(key.utf8))
    return JWT<MyClaims>.verify(signedJWT, using: jwtVerifier)
}

private func decodeJWT(key:String,encodedJWT:String)-> String{
    let jwtVerifier = JWTVerifier.hs256(key:  Data(key.utf8))
    let jwtDecoder = JWTDecoder(jwtVerifier: jwtVerifier)
    let jwt = try? jwtDecoder.decode(JWT<MyClaims>.self, fromString: encodedJWT)
    let json = ["data":jwt?.claims.data ?? "Error Decode JWT token"] as [String : Any]
    let jsonData = try! JSONSerialization.data(withJSONObject: json, options: JSONSerialization.WritingOptions.prettyPrinted)
    let jsonString = NSString(data: jsonData, encoding: String.Encoding.utf8.rawValue) as? String
    return jsonString ?? "Error Decode JWT Token"
}

struct MyClaims: Claims {
    let data: String
//    let exp: Date
}
extension Date {
    var millisecondsSince1970: Int64 {
        Int64((self.timeIntervalSince1970 * 1000.0).rounded())
    }

    init(milliseconds: Int64) {
        self = Date(timeIntervalSince1970: TimeInterval(milliseconds) / 1000)
    }
}
