import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

import 'jwtson_platform_interface.dart';
/*
 * Dayona
 * Created on Thu Jun 02 2022
 * email : dayonaart@gmail.com
 * Copyright (c) 2022
 */

/// An implementation of [JwtsonPlatform] that uses method channels.
class MethodChannelJwtson extends JwtsonPlatform {
  /// The method channel used to interact with the native platform.
  @visibleForTesting
  final methodChannel = const MethodChannel('jwtson');

  @override
  Future<String?> generateToken(String? key, String? data) async {
    final result = await methodChannel
        .invokeMethod<String>('generatePayload', <String, dynamic>{'key': key, 'data': data});
    return result;
  }

  @override
  Future<bool?> verifyJwtToken(String? key, String? data) async {
    final result = await methodChannel
        .invokeMethod<bool>('verifyJwtToken', <String, dynamic>{'key': key, 'data': data});
    return result;
  }

  @override
  Future<PayloadResult?> decodeJwtToken(String? key, String? data) async {
    final result = await methodChannel
        .invokeMethod<String?>('decodeJWT', <String, dynamic>{'key': key, 'data': data});
    print(result);
    try {
      return PayloadResult.fromJson(jsonDecode(result!));
    } catch (e) {
      print(e);
      return null;
    }
  }
}

class PayloadResult {
  String? data;
  // var expired;

  PayloadResult({
    this.data,
    // this.expired,
  });

  factory PayloadResult.fromJson(Map<String, dynamic> json) => PayloadResult(
        data: json['data'],
        // expired: json['date'],
      );

  Map<String, dynamic> toJson() => {
        'data': data,
        // 'expired': expired,
      };
}
